package glog;

import java.applet.*;
import java.awt.*;
import java.io.*;
import java.net.*;

import prolog.*;

/**
 * GLog applet/application.
 * <P>
 * Details of usage can be found in the accompanying docs.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: GLog.java,v 1.5 2000/08/11 13:37:32 peteg Exp $
 */
public class GLog extends Applet {
    /* Standalone constants. */
    private final static int WIDTH  = 400;
    private final static int HEIGHT = 400;

    private Dimension dim;
    private Graphics offGraphics;
    private Image offImage;

    private static final String PdbURL = "dbURL";

    private Prolog prolog;

//     private boolean applet = true;

    public GLog() {
	super();
    }

    public String getAppletInfo() {
	return("GLog Applet - Peter Gammie, 1999.");
    }

    public String[][] getParameterInfo() {
	String info[][] = {
	    {PdbURL,	"URL",		"URL of the serialized database of the drawing (contains \"draw/1\")."}
	};

	return(info);
    }

    public Dimension getMinimumSize() {
        return(getPreferredSize());
    }

    public Dimension getPreferredSize() {
        return(new Dimension(WIDTH, HEIGHT));
    }

    public int getHeight() {
	return(dim.height);
    }

    public int getWidth() {
	return(dim.width);
    }

    private void setup() {
	dim = getSize();
	offImage = createImage(dim.width, dim.height);
// 	System.err.println("dim: " + dim.width + " " + dim.height);
	offGraphics = offImage.getGraphics();
	clear();
    }

    public void init() {
	String dbURL;
	String s;
	URL documentBase = getDocumentBase();

	if((dbURL = getParameter(PdbURL)) == null)
	    throw new RuntimeException("GLog: must specify: " + PdbURL);

	try {
	    /* Fetch the source file. */
	    showStatus("GLog: loading database.");

	    URL url = new URL(documentBase, dbURL);
            ObjectInputStream in = new ObjectInputStream(url.openStream());
	    Database db = (Database)in.readObject();

	    prolog = new Prolog(db);
	    prolog.obj = this;

	    in.close();
	} catch (IOException e) {
	    e.printStackTrace();
	    throw new RuntimeException("GLog: error loading: " + dbURL + " (IOException)");
	} catch (ClassNotFoundException e) {
            /* Really spit the dummy. */
            System.err.print("GLog(database): class not found: ");
            e.printStackTrace();
            throw new RuntimeException("GLog: ClassNotFoundException.");
// 	}
	} catch(Throwable e) {
	    System.err.println(e);
	}

	/* Setup the graphics buffer. */
	setup();
    }

    public void destroy() {
    }

    public void start() {
	showStatus("GLog running.");

	try {
	    /* Draw the picture. */
	    prolog.find(prolog.toGoal(new Compound("draw")));
	} catch(PrologException e) {
	    e.printStackTrace();
	}
    }

    public void stop() {
    }

    public void clear() {
	offGraphics.setColor(Color.lightGray);
	offGraphics.fillRect(0, 0, dim.width, dim.height);
	offGraphics.setColor(Color.black);
    }

    public void drawLine(int[] x, int[] y) {
// 	for(int i = 0; i < x.length; i++) {
// 	    System.err.println("GLog.drawLine: x[" + i + "] = " + x[i] + " y[" + i + "] = " + y[i]);
// 	}

	offGraphics.drawPolyline(x, y, x.length);
	update(this.getGraphics());
    }

    public void paint(Graphics g) {
	update(g);
    }

    public void update(Graphics g) {
	g.drawImage(offImage, 0, 0, this);
    }

    /**
     * Standalone entry point. Creates a new graphics context and prolog
     * thread, then drops into the read-eval loop.
     */
    public static void main(String[] argv) {
	GLog panel = new GLog();
	GLogWindow window = new GLogWindow(panel);

        window.pack();
	panel.setup();
        window.setVisible(true);

	try {
	    panel.prolog = Main.parseCommandTail(argv, panel);
	    Main.readeval(panel.prolog);
	} catch(PrologException e) {
	    e.printStackTrace();
	}
    }
}
