package prolog;

import prolog.util.Vector;

/**
 * Representation of bindings - variables that aren't bound (to ground
 * terms) by the unification of the goal with the unifying clause.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: Binding.java,v 1.22 2000/11/16 10:30:07 peteg Exp $
 */
public final class Binding extends Variable {
    /**
     * Used to give each object a unique tag for debugging purposes.
     */
    private static int globalVarID = 0;

    /**
     * Value of the global counter at the time this var was created.
     */
    private int gvarid;

    private Term binding;

    /**
     * The next <TT>Binding</TT> in the trail.
     */
    Binding next;

    public Binding(int varid) {
	super(varid);
	gvarid = globalVarID++;

//  	System.err.println("new Binding(): " + gvarid);
    }

    public void bind(Term t) {
// 	if(binding == null) {
// 	    System.err.println("Binding.bind(): " + this + " to: " + t);
// 	    Thread.dumpStack();
	    binding = t;
// 	} else {
// 	    throw new RuntimeException("Binding.bind(): " + this + " not free. (arg: " + t + ")");
// 	}
    }

    public void unbind() {
//   	if(binding != null) {
//  	    System.err.println("unbind: " + this);
 	    binding = null;
//   	} else
//   	    throw new RuntimeException("Binding.unbind(): " + this + " not bound.");
    }

    public Term getBinding() {
// 	System.err.println("(" + gvarid + ")");
	return (binding == null) ? this : binding.getBinding();
    }

    public Term instantiate(Term[] insts) {
	return (binding == null) ? this : binding.getBinding();
    }

    public boolean unify(Term t, Term[] insts, Prolog p) {
//   	System.err.println("Binding.unify(): this = " + this + " t = " + t);

	if(binding != null) {
	    return binding.unify(t, insts, p);
	}

// 	System.err.println("Binding.unify(): binding");

	binding = t.instantiate(insts);
	p.push(this);
	return true;
    }

    public boolean identical(Term t) {
	return (binding == null)
	    ? this == t.getBinding()
	    : binding.identical(t);
    }

    public Term normalise_copy(Variable[] vars,
			       int next_new_var,
			       Vector new_vars) {
	throw new RuntimeException();
    }

    public void typify(TypeDictionary types, boolean map) {
	if(binding == null) {
	    ti = new TermInfo(new TypeVariable(0, true), new TermInfo[1]);
	} else {
// 	    System.err.println("Binding.typify(" + this + "): -> bound.");
// 	    throw new RuntimeException();
 	    binding.typify(types, map);

	    this.ti = binding.ti;
	}
    }

    public String getBID() {
	char c = (char)('A' + (gvarid % 26));
	return "(b_" + c + (gvarid / 26) + ")" + (binding == null ? "" : binding.toString());
    }

    public String displayString() {
  	if(binding == null) {
	    char c = (char)('A' + (gvarid % 26));
	    return "b_" + c + (gvarid / 26);
   	} else {
     	    return binding.displayString();
//    	    return "(" + gvarid + ")" + binding.displayString();
	}
    }

    public String toString() {
  	if(binding == null) {
	    char c = (char)('A' + (gvarid % 26));
	    return "b_" + c + (gvarid / 26);
   	} else {
     	    return binding.toString();
//     	    return "(" + gvarid + ")" + binding.toString();
	}
    }
}
