package prolog;

import java.io.*;

/**
 * An entry in a <TT>ClauseIndex</TT>.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: ClauseIndexRec.java,v 1.6 2000/11/12 12:22:44 peteg Exp $
 */
class ClauseIndexRec implements Cloneable, Serializable {
    /**
     * The list of clauses that form this predicate.
     */
    protected ClauseList firstClause;

    /**
     * The last clause in this predicate, to speed insertions at the end.
     *
     * @see prolog.lib.assertz_1
     */
    protected ClauseList lastClause;

    ClauseIndexRec next;

    ClauseIndexRec() {
    }

    /**
     * Adds a clause to the beginning of the predicate.
     */
    protected void puta(Clause c) {
	firstClause = new ClauseList(c, firstClause);

	if(lastClause == null) {
	    lastClause = firstClause;
	}
    }

    /**
     * Adds a clause to the end of the predicate.
     */
    protected void putz(Clause c) {
// 	System.err.println("CIR.putz(): " + c + " -> CIR before:\n" + this + "-----------");

	if(lastClause == null) {
	    firstClause = lastClause = new ClauseList(c);
	} else {
	    lastClause = lastClause.next = new ClauseList(c);
	}

// 	System.err.println(" after:\n" + this + "-----------");
    }

    /**
     * Removes all clauses from this index.
     */
    public void clear() {
	firstClause = lastClause = null;
    }

    /**
     * Removes a clause from this index.
     */
    void remove(Clause c) {
	ClauseList cur, prev = null;

	for(cur = firstClause; cur != null; prev = cur, cur = cur.next) {
	    if(cur.clause == c) {
		if(prev == null) {
		    firstClause = firstClause.next;
		    if(firstClause == null) {
			lastClause = null;
		    }
		} else {
		    prev.next = cur.next;
		    if(prev.next == null) {
			lastClause = prev;
		    }
		}

		return;
	    }
	}

	throw new RuntimeException("ClauseIndex.remove(): clause not found: " + c);
    }

    /**
     * Copies just this <TT>ClauseIndexRec</TT> node and it's associated
     * <TT>ClauseList</TT> nodes.
     */
    protected Object clone() {
	try {
	    ClauseIndexRec cir = (ClauseIndexRec)super.clone();

	    cir.clear();

	    for(ClauseList cl = firstClause; cl != null; cl = cl.next) {
		cir.putz(cl.clause);
	    }

	    return cir;
	} catch(CloneNotSupportedException e) {
	    throw new InternalError("ClauseIndexRec.clone()");
	}
    }

    public String toString() {
	return firstClause == null ? " < null > " : firstClause.toString();
    }
}
