package prolog;

/* The JavaClass package. */
import de.fub.bytecode.classfile.*;
import de.fub.bytecode.generic.*;

import java.io.*;

/**
 * Represents a <TT>functor/arity</TT> pair. There should be precisely one
 * instance of this class per such pair.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: FunctorArity.java,v 1.5 2000/11/12 12:22:44 peteg Exp $
 */
public class FunctorArity implements Serializable {
    static final String ClassName    = "prolog.FunctorArity";
    static final ObjectType JavaType = new ObjectType("prolog/FunctorArity");

    public String functor;
    public int arity;

    /**
     * Used by the clause indexer, functor/arity comparsions, compiled
     * disjunctions.
     */
    int functor_arity;

    private static int gfa = 0;

    /**
     * Used by the type subsystem.
     *
     * @see prolog.TypeDictionary
     */
    AlgebraicType type;
    CompoundType component;

    FunctorArity next;

    FunctorArity(String functor, int arity, int functor_arity) {
	this.functor       = functor;
	this.arity         = arity;
	this.functor_arity = functor_arity;

// 	System.err.println("FunctorArity: " + functor + "/" + arity + " -> " + functor_arity);
    }

    FunctorArity(String functor, int arity) {
	this(functor, arity, gfa++);
    }

    public String toString() {
	StringBuffer out = new StringBuffer();

	out.append("FA: ");
	out.append(functor_arity);
	out.append(" ");

	if(type != null) {
	    type.toStringDef(out);
	} else {
	    out.append(functor);
	    out.append("/");
	    out.append(arity);
	}

 	return out.toString();
    }

    /**
     * Object serialisation breaks the String.intern() method, which we rely
     * on to speed up functor equality checks (pointer equality v's string
     * equality). So we fix that up here...
     */
    private void readObject(ObjectInputStream in)
	throws IOException, ClassNotFoundException {
	in.defaultReadObject();
	functor = functor.intern();
    }
}
