package prolog;

import prolog.util.Vector;

/**
 * <TT>'->'/2</TT>: Represents an if-then goal in the body of a clause.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: IfThen.java,v 1.10 2000/10/02 12:09:33 peteg Exp $
 */
public class IfThen extends Goal {
    public static final String FUNCTOR = "->".intern();
    public static final int    ARITY   = 2;

    Goal if_goal;
    Goal then_goal;

    IfThen(Goal if_goal, Goal then_goal) {
	this.if_goal   = if_goal;
	this.then_goal = then_goal;
    }

    public boolean unify(Term[] insts, Prolog p) throws PrologException {
	if_goal.instantiate(insts);

	return p.once(if_goal) && then_goal.unify(insts, p);
    }

    public boolean redo(Term[] insts, Prolog p, ChoicePoint cp) throws PrologException {
	throw new RuntimeException("IfThen.redo()");
// 	return unifier.redo(g.instantiateHead(insts), p, cp);
    }

    /**
     * Instantiate a list of Goals with the given substitutions. We
     * overwrite the old <TT>Compound</TT> objects in the list with their
     * instantiated counterparts.
     */
    void instantiate(Term[] insts) {
	if_goal.instantiate(insts);
	then_goal.instantiate(insts);
    }

    Goal normalise(Variable[] vars,
		   int next_new_var,
		   Vector new_vars,
		   TypeDictionary types) {
	throw new RuntimeException("IfThen.normalise()");
    }

    public String toString() {
	StringBuffer out = new StringBuffer();

	out.append("(");
	out.append(if_goal.toString());
	out.append(" -> ");
	out.append(then_goal.toString());
	out.append(")");

	return out.toString();
    }
}
