package prolog;

import java.io.*;

/**
 * Represents a predicate that will be interpreted (i.e. is an ordered list
 * of <TT>Clause</TT>s). Clause management is provided by
 * <TT>ClauseIndex</TT> classes.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: InterpretedPredicate.java,v 1.11 2000/11/07 12:04:56 peteg Exp $
 * @see prolog.ClauseIndex
 */
public class InterpretedPredicate extends Predicate {
    /**
     * The index of clauses for this predicate.
     *
     * @see prolog.lib.listdb_1
     */
    public ClauseIndex index;

    public InterpretedPredicate(int arity) {
	super();
//   	this.index = arity > 0 ? new ClauseIndexUF() : new ClauseIndex();
    	this.index = arity > 0 ? new ClauseIndexTrad() : new ClauseIndex();
//   	this.index = new ClauseIndex();
    }

    public InterpretedPredicate(ClauseIndex index) {
	super();
	this.index = index;
    }

    public boolean unify(Term[] args, Prolog p) throws PrologException {
	return index.first(args, p);
    }

    public boolean redo(Term[] args, Prolog p, ChoicePoint cp) throws PrologException {
	return index.redo(args, p, cp);
    }

    public void puta(Clause c) {
	index.puta(c);
    }

    public void putz(Clause c) {
	index.putz(c);
    }

    public String toString() {
	ClauseList cl;
	StringBuffer out = new StringBuffer("\n");

	if((cl = index.getClauses()) != null) {
	    out.append(cl.toString());
	}

 	return out.toString();
    }
}
