package prolog;

import prolog.math.*;

/* The JavaClass package. */
import de.fub.bytecode.classfile.*;
import de.fub.bytecode.generic.*;

/**
 * Constants shared amongst the classes doing code generation.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: JITDefs.java,v 1.15 2000/11/13 11:55:04 peteg Exp $
 * @see prolog.ClauseCodeGen
 * @see prolog.PredicateCodeGen
 */
public class JITDefs implements de.fub.bytecode.Constants {
    /* This class is non-instantiable. */
    private JITDefs() {
    }

    /* Class name constants. */
    public static final String package_name = "prolog.lib";
    public static final String extends_class_name = "prolog.CompiledPredicate";

    public static final String predicate_class_name = "prolog.Predicate";
    public static final Type predicate_type = new ObjectType(predicate_class_name);

    /* The name and type of the constructor in <TT>CompiledPredicate</TT>. */
    public static final String pred_init_name = "<init>";
    public static final Type pred_init_ret_type = Type.VOID;
    public static final Type[] pred_init_arg_type = Type.NO_ARGS;

    /* The type of a term array. */
    public static final ArrayType term_array_type = new ArrayType(TermType.JavaType,
								  1);

    /**
     * The location of the args array.
     */
    public static final short args_lv_index = 1;
    public static final LocalVariableGen args_lv = new LocalVariableGen(args_lv_index,
									"args",
									term_array_type,
									null,
									null);
    public static final Instruction args_load_inst = InstructionConstants.ALOAD_1;

    public static final String prolog_class_name = "prolog/Prolog";
    public static final ObjectType prolog_type = new ObjectType(prolog_class_name);

    /**
     * The location of the Prolog object.
     */
    public static short prolog_lv_index = 2;
    public static LocalVariableGen prolog_lv = new LocalVariableGen(prolog_lv_index,
						      "prolog",
						      prolog_type,
						      null, null);
    public static Instruction prolog_load_inst = InstructionConstants.ALOAD_2;

    /**
     * The first free <TT>LocalVariable</TT> following the arguments.
     */
//     public static short first_free_lv = 3;
    public static short first_free_lv = 4;

    /**
     * The name and type of the method that does the work.
     */
//     public static String pred_unify_name = "unify";
    public static String unify_name = "exec";
    public static Type unify_ret_type = Type.BOOLEAN;
    public static Type[] unify_arg_type = {term_array_type,
					   prolog_type};
    public static String[] unify_arg_names = {"args", "p"};

    public static String prologException_name = "prolog.PrologException";

    /**
     * The maximum stack depth of the <TT>unify()</TT> method. Should be
     * dynamically determined or estimated.
     */
    public static int unify_stack_depth = 20;

    public static ObjectType choicePoint_type = new ObjectType("prolog/ChoicePoint");


    /* prolog.Term definitions. */
    public static String term_getBinding = "getBinding";
    public static ObjectType term_getBinding_ret_type = TermType.JavaType;
    public static Type[] term_getBinding_args_type = Type.NO_ARGS;

    /* prolog.Binding definitions. */
    public static String binding_unbind = "unbind";
    public static Type binding_unbind_ret_type = Type.VOID;
    public static Type[] binding_unbind_args_type = Type.NO_ARGS;

    /* prolog.math.Numeric definitions. */
    public static String intNum_class = "prolog.math.IntNum";

    public static String intNum_add = "add";
    public static ObjectType intNum_add_ret_type = IntType.JavaType;
    public static Type[] intNum_add_args_type = {IntType.JavaType};

    public static String numeric_compare = "compare";
    public static Type numeric_compare_ret_type = Type.INT;
    public static Type[] numeric_compare_args_type = {NumericType.JavaType};

    public static String intNum_make = "make";
    public static ObjectType intNum_make_ret_type = IntType.JavaType;
    public static Type[] intNum_make_args_type = {Type.INT};

    public static String intNum_sub = "sub";
    public static ObjectType intNum_sub_ret_type = IntType.JavaType;
    public static Type[] intNum_sub_args_type = {IntType.JavaType};

    public static String intNum_valueof = "valueOf";
    public static ObjectType intNum_valueof_ret_type = IntType.JavaType;
    public static Type[] intNum_valueof_args_type = {Type.STRING};

    /* prolog.Prolog definitions. */
    public static String prolog_push = "push";
    public static Type prolog_push_ret_type = Type.VOID;
    public static Type[] prolog_push_args_type = {TypeVariable.JavaType,
						  TermType.JavaType};


    // ???
    public static String call_name = "unify";
    public static Type call_ret_type = Type.BOOLEAN;
    public static Type[] call_arg_type = {term_array_type, prolog_type};
    public static String[] call_arg_names = {"args", "prolog"};

    /* prolog.Compound definitions. */

    /* functor_arity field. */
    public static String compound_functor_arity_name = "functor_arity";
    public static Type   compound_functor_arity_type = Type.INT;

    /* constructor for arity > 0 */
    public static String compound_new = "<init>";
    public static Type compound_new_ret_type = Type.VOID;
    public static Type[] compound_new_args_type = new Type[]{FunctorArity.JavaType,
							     term_array_type};
    public static Type[] compound_new_no_args_type = new Type[]{FunctorArity.JavaType};
}
