package prolog;

/**
 * These predicates are compilation candidates.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: JITPredicate.java,v 1.31 2000/11/16 10:30:07 peteg Exp $
 * @see prolog.PredicateCodeGen
 */
public class JITPredicate extends InterpretedPredicate {
    /**
     * A record of the types of the arguments that have been unified with
     * this predicate. Maps <TT>TermType -> TypeRec</TT>.
     */
    private PredicateTypeDictionary types;

    public JITPredicate(int arity) {
	super(arity);
	types = new PredicateTypeDictionary();
    }

    public boolean unify(Term[] args, Prolog p) throws PrologException {
	Predicate c;

// 	System.err.println("JITPredicate.unify(): "
// 			   + functor + "/" + arity
// 			   + " args: ");
// 	if(args != null) {
// 	    for(int i = 0; i < args.length; i++) {
// 		System.err.println("   " + args[i]);
// 	    }
// 	} else {
// 	    System.err.println("   < none >");
// 	}

	if((c = types.called(this, args, p)) == null) {
//  	    System.err.println("JITPredicate.unify(): using interpreted code: " + functor + "/" + arity);
	    return super.unify(args, p);
	} else {
//  	    System.err.println("JITPredicate.unify(): using compiled code: " + functor + "/" + arity);
	    return c.unify(args, p);
	}
    }

//     public boolean redo(Term[] args, Prolog p, ChoicePoint cp) throws PrologException {
// 	Predicate c;

// 	if((c = types.called(this, args, p)) == null) {
// 	    return super.redo(args, p, cp);
// 	} else {
// 	    return c.redo(args, p, cp);
// 	}
//     }

    public String toString() {
	StringBuffer out = new StringBuffer();

	out.append(super.toString());
	out.append(types.toString());
	out.append("% -- end of predicate --\n");
	return out.toString();
    }
}
