package prolog;

/**
 * Represents an operator.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: Operator.java,v 1.5 2000/08/17 13:01:23 peteg Exp $
 * @see prolog.OperatorTable
 */
class Operator {
    /* --- CONSTANTS --- */

    /**
     * The minimum precedence an operator can have.
     */
    static final int min_precedence = 0;

    /**
     * The maximum precedence an operator can have.
     */
    static final int max_precedence = 1200;

    static final int max_arg_of_functor_prec = 999;

    /**
     * This is a prefix operator.
     */
    static final int prefix = 0;

    /**
     * This is a postfix operator.
     */
    static final int postfix = 1;

    /**
     * This is an infix operator.
     */
    static final int infix = 2;

    /*
     * Operator types.
     */
    static final int FX  = 0;
    static final int FY  = 1;

    static final int XF  = 2;
    static final int YF  = 3;

    static final int XFX = 4;
    static final int XFY = 5;
    static final int YFX = 6;

    /*
     * Operator types as Strings.
     */
    static final String S_FX  = "fx";
    static final String S_FY  = "fy";

    static final String S_XF  = "xf";
    static final String S_YF  = "yf";

    static final String S_XFX = "xfx";
    static final String S_XFY = "xfy";
    static final String S_YFX = "yfx";

    /**
     * The string representation of this operator.
     */
    String lexeme;

    /**
     * The ISO-prolog name of this operator. We use this as the functor of
     * the resultant <TT>Compound</TT> term so that we can use Java's
     * dynamic linking (which have a restricted name space).
     */
    String name;

    /**
     * Pre-/In-/Post-fix.
     */
    int type;

    /**
     * Precedence of this operator.
     */
    int precedence;

    /**
     * Left precedence of this operator.
     */
    int left;

    /**
     * Right precedence of this operator.
     */
    int right;

    /**
     * The next operator with this name (i.e. lexeme).
     */
    Operator next;

    Operator(String lexeme, int type,
	     int precedence, int left, int right) {
	this(lexeme, lexeme, type, precedence, left, right);
    }

    Operator(String lexeme, String name, int type,
	     int precedence, int left, int right) {
	this.lexeme     = lexeme;
	this.name       = name;
	
	this.type       = type;

	this.precedence = precedence;
	this.left       = left;
	this.right      = right;
    }

    public String toString() {
 	StringBuffer out = new StringBuffer();

	out.append("\"");
	out.append(lexeme);
	out.append("\":\t");

	switch(type) {
	case prefix:
	    out.append("prefix");
	    break;
	case infix:
	    out.append("infix");
	    break;
	case postfix:
	    out.append("postfix");
	    break;
	default:
	    out.append("<< unknown >>");
	    break;
	}

	out.append("\t");
	out.append(precedence);

	if(name != lexeme) {
	    out.append("\t\"");
	    out.append(name);
	    out.append("\"");
	}

 	return out.toString();
    }
}
