package prolog;

import java.io.Serializable;

/* The JavaClass package. */
import de.fub.bytecode.classfile.*;
import de.fub.bytecode.generic.*;

/**
 * The base class of all types. This type represents the general case:
 * <TT>unknown</TT>.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: TermType.java,v 1.24 2000/11/16 10:30:08 peteg Exp $
 * @see prolog.PredicateCodeGen
 * @see prolog.JITPredicate
 */
public abstract class TermType implements Serializable {
    static final String ClassName    = "prolog.Term";
    static final ObjectType JavaType = new ObjectType("prolog/Term");

    protected TermType() {
    }

    ObjectType getJavaType(TermInfo ti) {
// 	System.err.println("TermType.getJavaType(): " + this);
	return JavaType;
    }

    /**
     * Chase reference chains.
     */
    public TermInfo getBinding(TermInfo arg0) {
	return arg0;
    }

    /**
     * Instantiates this <TT>TermType</TT> against a given
     * <TT>Term</TT>.
     */
    abstract TermInfo instantiate(Term t, TermInfo[] typevars,
				  TypeDictionary types, boolean map);

    /**
     * Unifies two <TT>TermType</TT> objects. We use the PTD as a trail.
     *
     * @see prolog.PredicateTypeDictionary#called
     */
    abstract boolean unify(TermInfo ti,
			   TermInfo arg,
			   TypeTrail trail);

    /**
     * Generates code to unify one <TT>Term</TT> with another. The idea is
     * we invoke this method on the left/first/arg0 <TT>TermType</TT>
     * object, as it is always a <TT>Variable</TT>.
     */
    public abstract boolean compile_unify(ClauseCodeGen ccg,
					  TermInfo arg0, TermInfo arg1);

    /**
     * Loads a <TT>Term</TT>.
     */
    public void load(ClauseCodeGen ccg, TermInfo arg0, boolean cast) {
	TermInfo ti = arg0; //= getBinding(arg0);

// 	if(ti != arg0) {
// 	    throw new RuntimeException();
// 	}

	if(ti.location == null) {
	    // Create it.
// 	    System.err.println("TermType.load(): creating term: " + this + " should cast: " + cast);
	    create(ccg, ti);
	} else {
	    // Load it
// 	    System.err.println("TermType.load(): loading term: " + this + " should cast: " + cast);
	    ti.location.load(ccg, cast ? getJavaType(ti) : null);
	}
    }

    /**
     * Creates the <TT>Term</TT> denoted by this <TT>TermType</TT>.
     *
     * Stack: <TT>... -> ..., TermRef</TT>, where <TT>TermRef</TT> has the
     * type of the created <TT>Term</TT>.
     */
    public abstract void create(ClauseCodeGen ccg, TermInfo arg0);

    public void toString(StringBuffer out) {
	out.append("TermType");
    }

    public final String toString() {
	StringBuffer out = new StringBuffer();

	toString(out);

	return out.toString();
    }
}
