package prolog.lib;

import java.io.*;

import prolog.*;

/**
 * <TT>deserialize/1</TT>: Read a canned database from the specified
 * file. Note the existing db gets nuked.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: deserialize_1.java,v 1.5 2000/08/02 11:47:54 peteg Exp $
 * @see prolog.Database
 * @see prolog.lib.serialize_1
 */
public class deserialize_1 extends CompiledPredicate {
    public deserialize_1() {
    }

    public boolean exec(Term[] args, Prolog p) throws ArgumentException {
//    	System.err.println("deserialize: " + args);

	try {
	    Compound arg0 = (Compound)args[0].getBinding();
	    String filename = arg0.functor;

	    /* Standard technique - straight out of "Java in a Nutshell" */
	    FileInputStream fos = new FileInputStream(filename);
	    ObjectInputStream in = new ObjectInputStream(fos);

	    p.db = (Database)in.readObject();
	    in.close();

	    return true;
	} catch (ClassCastException e) {
	    throw new ArgumentException("deserialize/1: invalid argument: ", args);
	} catch (IOException e) {
	    System.err.println("deserialize/1: error reading from file.");
	    e.printStackTrace();
	    throw new InternalError("serialize/1: error reading from file.");
	} catch (ClassNotFoundException e) {
	    System.err.print("deserialize/1: ");
	    e.printStackTrace();
	    throw new InternalError("deserialize/1: ClassNotFoundException.");
	}
    }
}
