package prolog.lib;

import java.io.*;

import prolog.*;
import prolog.math.Numeric;

/**
 * <TT>op/3</TT>: Add/change operators dynamically.
 * <P>
 * <TT>op(Priority, Op_specifier, Operator)</TT>
 * <P>
 * A <TT>Priority</TT> of 0 removes an operator property from an atom.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: op_3.java,v 1.4 2000/08/10 12:51:04 peteg Exp $
 * @see prolog.Operator
 * @see prolog.OperatorTable
 * @see prolog.Parser
 */
public class op_3 extends CompiledPredicate {
    public op_3() {
    }

    /**
     * @exception prolog.PrologException on invalid precedence.
     */
    public boolean exec(Term[] args, Prolog p) throws PrologException {
// 	System.err.println("op/3");

	try {
	    int priority        = ((Numeric)args[0]).intValue();
	    String op_specifier = ((Compound)args[1]).functor;
	    String operator     = ((Compound)args[2]).functor;

	    p.parser.ops.put(priority, op_specifier, operator);

	    return true;
	} catch (ClassCastException e) {
	    throw new ArgumentException("op/3: invalid argument: ", args);
	}
    }
}
