package prolog.lib;

import prolog.*;

/**
 * <TT>unify/2</TT>: Unify two terms.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: unify_2.java,v 1.20 2000/11/12 12:22:45 peteg Exp $
 */
public class unify_2 extends CompiledPredicate {
    /**
     * Constant "unify" functor used when normalising.
     *
     * @see prolog.JITPredicate
     */
    public static final String functor = "unify".intern();

    /**
     * An instance of this class used when normalising.
     */
    public static final unify_2 unify = new unify_2();

    public unify_2() {
    }

    public boolean exec(Term[] args, Prolog p) {
// 	System.err.println("unify/2 call: " + args[0] + ", " + args[1]);

	if(args[0] == null || args[1] == null) {
	    throw new RuntimeException("unify/2: " + args[0] + ", " + args[1]);
	}

	boolean success = args[0].unify(args[1], null, p);

// 	System.err.println("unify/2 ret : " + args[0] + ", " + args[1] + "(" + success +")");
	return success;
    }

    public boolean compile_call(ClauseCodeGen ccg, Term[] args, boolean lastcall) {
	// No need for getBinding() -- no Term binding chains.
	Term arg0 = args[0];
	Term arg1 = args[1];

	TermInfo arg0_ti = arg0.ti;
	TermInfo arg1_ti = arg1.ti;

// 	arg0_ti = arg0_ti.type.getBinding(arg0_ti);
// 	arg1_ti = arg1_ti.type.getBinding(arg1_ti);

//  	System.err.println("unify_2.compile_unify(\n\t"
// 			   + arg0 + " TI: " + arg0_ti
// 			   + ",\n\t"
// 			   + arg1 + " TI: " + arg1_ti
// 			   + ")");

	boolean succ = arg0_ti.type.compile_unify(ccg, arg0_ti, arg1_ti);

//  	System.err.println("--- finished ---\nunify_2.compile_unify(\n\t"
// 			   + arg0 + " TI: " + arg0_ti
// 			   + ",\n\t"
// 			   + arg1 + " TI: " + arg1_ti
// 			   + ")\n--- "
// 			   + succ);

	return succ;
    }
}
