package prolog.lib;

import prolog.*;

import prolog.util.Vector;

/**
 * <TT>univ/2</TT> ("=.."): Build and pull apart <TT>Compound</TT> terms.
 * <P>
 * <TT>Term =.. [Functor | Args]</TT>
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: univ_2.java,v 1.7 2000/09/15 15:11:35 peteg Exp $
 */
public class univ_2 extends CompiledPredicate {
    public univ_2() {
    }

    public boolean exec(Term[] args, Prolog p) throws ArgumentException {
//      	System.err.println("univ/2 (=..) entry: " + t);

	try {
	    Term arg0 = args[0].getBinding();
	    Term arg1 = args[1].getBinding();

	    /* If the second term is ground, then build a new Compound. It's
               cheaper than building a new list. */
	    if(!(arg1 instanceof Binding)) {
		/* functor(arg1, ..., argN) <- list */
		Compound list = (Compound)arg1;

		String functor = ((Compound)list.args[0].getBinding()).functor;
		Vector as = new Vector();

		list = (Compound)list.args[1].getBinding();
		while(list.functor != Parser.empty_list_functor) {
//  		    System.err.println("univ/2: list = " + list);
		    as.addElement(list.args[0]);
		    list = (Compound)list.args[1].getBinding();
		}

		/* Create a new Compound. */
		Term[] arglist = null;

		if(as.size() > 0) {
		    args = new Term[as.size()];
		    as.trimToSize();
		    as.copyInto(arglist);
		}

		return arg0.unify(new Compound(functor, arglist), null, p);
	    } else {
		/* functor(arg1, ..., argN) -> list */
		Compound c = (Compound)arg0;
		Compound list = new Compound(Parser.empty_list_functor, null);

		for(int i = c.arity; i-- > 0; ) {
		    list = new Compound(Parser.list_functor,
					new Term[]{c.args[i],
						   list});
		}

		list = new Compound(Parser.list_functor,
				    new Term[]{new Compound(c.functor),
					       list});

// 		System.err.println("univ/2: list = " + list);
		return arg1.unify(list, null, p);
	    }
	} catch (ClassCastException e) {
	    throw new ArgumentException("univ/2: invalid arguments: ", args);
	}
    }

    public String toString() {
	return "'=..'";
    }
}
