package prolog;

import java.io.ObjectStreamException;

/**
 * Represents the empty list / end-of-list <TT>Compound</TT> term
 * (<TT>[]</TT>). Note that there should only ever be one instance of this
 * class created, so that a pointer-equality check is sufficient to
 * determine unifiability.
 *
 * @author Peter Gammie (<A HREF="MailTo:peteg@cse.unsw.edu.au">peteg@cse.unsw.edu.au</A>)
 * @version $Id: EmptyList.java,v 1.1 2000/08/06 13:43:46 peteg Exp $
 */
public class EmptyList extends Compound {
    /**
     * Represents the end-of-list functor.
     */
    public final static String emptyListFunctor = "[]".intern();

    /**
     * Represents the end-of-list <I>Term</I>.
     */
    public final static EmptyList emptyList = new EmptyList();

    private EmptyList() {
 	super(emptyListFunctor);
    }

    public boolean unify(Term t, Term[] insts, Prolog s) {
// 	if(t instanceof Variable) {
// 	    return t.unify(this, insts, s);
// 	}

	return this == t.getBinding() || t.unify(this, insts, s);
    }

    public boolean identical(Term t) {
	return this == t.getBinding();
    }

    /**
     * Enforce the singleton pattern when we get deserialised.
     */
    private Object readResolve() throws ObjectStreamException {
	return emptyList;
    }

    public String displayString() {
	return emptyListFunctor;
    }

    public String toString() {
	return emptyListFunctor;
    }
}
