package prolog.pred;

import prolog.*;

/**
 * append/3: Concatenate two lists. (test backtracking)
 *
 * @author Peter Gammie
 */

public class append_3 extends Pred {
    public append_3() {
	functor = "append";
	arity = 3;
	usecp = true;
    }

    public boolean unify(Term t, Prolog s) {
	int clause = s.cp.clausenum;
	Structured st = (Structured)t;

	// Grab a copy of the  arguments if we haven't already
	if(st.tmp_args == null) {
// 	    System.out.println("Copying args -- " + st.args[0] + " -- " + st.args[1]);
	    st.tmp_args = new Term[st.args.length];
	    for(int i = 0; i < st.args.length; i++)
		st.tmp_args[i] = st.args[i];
	}

	Term arg0 = st.tmp_args[0];
	Term arg1 = st.tmp_args[1];
	Term arg2 = st.tmp_args[2];

// 	System.out.println("append entry: " + clause);

	while(true) {
	    switch(clause) {
	    // append(X, Y, Z) :- unify(X, []), unify(Y, Z).
	    case 0: {
// 		System.out.println("append - clause0");
// 		for(int i = 0; i < st.args.length; i++)
// 		    System.out.println("arg" + i + ": " + st.tmp_args[i]);

		if(arg0.unify(new Structured("[]"), s)
		   && arg1.unify(arg2, s)) {
// 		    System.out.println("append0 - success");
// 		    for(int i = 0; i < st.args.length; i++)
// 			System.out.println("arg" + i + ": " + st.tmp_args[i]);
		    s.cp.clausenum = 1;
		    return(true);
		}
		s.backtrack();
	    }

	    // append(A, Y, C) :- unify(A, [X|Xs]), unify(C, [X|Z]),
	    //                    append(Xs, Y, Z).
	    case(1): {
// 		System.out.println("append - clause1");
// 		for(int i = 0; i < st.args.length; i++)
// 		    System.out.println("arg" + i + ": " + st.tmp_args[i]);

		Variable x = new Variable(null, 0);
		Variable xs = new Variable(null, 1);
		Variable z = new Variable(null, 2);
		Structured xxs = new Structured(".", new Term[]{x, xs});
		Structured xz = new Structured(".", new Term[]{x, z});

		if(arg0.unify(xxs, s)
		   && arg2.unify(xz, s)) {
// 		    System.out.println("[X|Xs] = " + xxs);
// 		    System.out.println("[X|Z] = " + xz);

		    st.tmp_args[0] = arg0 = xs;
		    st.tmp_args[2] = arg2 = z;
		    clause = 0;
 		    s.cp.bindingIndex = s.bindingStack; // Update the choicepoint to reflect 
		    continue;
		}
		s.backtrack();
	    }
	    default: {
		s.cp = null;
		st.tmp_args = null;
		return(false);
	    }
	    }
	}
    }
}
