package prolog.pred;

import prolog.*;

/**
 * call/1: Succeeds if it's argument is bound to a Compound term, and that
 * term (treated as a goal) has a solution, otherwise fails. This predicate
 * is highly dependent on the behaviour of the Prolog engine.
 *
 * @see prolog.Prolog
 * @author Peter Gammie
 */
public class call_1 extends Compiled {
    public call_1() {
	super("call", 1, true);
    }

    public boolean unify(Term t, Prolog s) {
	Compound arg0;
	ClauseList clause;
	Head unifier;
	Variable insts[];

	try {
	    arg0 = (Compound)((Compound)t).args[0].getBinding();
	} catch(ClassCastException e) {
	    System.err.println("call/1: invalid argument: " + t);
	    return(false);
	}

	for(clause = s.db.get(arg0); clause != null; clause = clause.next) {
	    unifier = (Head)clause.t;

	    /* Copy the head of the unifier */
	    insts = new Variable[unifier.numVars];
	    unifier = (Head)unifier.copy(s, insts);

//  	    System.err.println("call arg0: " + arg0 + " unifier: " + unifier);

	    if(unifier.unify(arg0, s)) {
		/* Success. Patch together a new goal list. Save a
		 * choicepoint if we need one. Change the current goal to
		 * our argument (so on backtracking the goal is correct).
		 *
		 * The Prolog engine is going to set the goal list to the
		 * next node on the list, so we insert the body of the
		 * unifier between us and the rest of the existing goal
		 * list.
		 */

//   		System.err.println("call: success.");

		if(clause.next != null) {
		    s.cp.clause = clause;
		    s.cp.goal = new ClauseList(arg0, s.goal.next);
//  		    System.err.println("call choicepoint: " + s.cp);
		} else
		    s.cp.finished();

		clause = ClauseList.copy(unifier.body, s.goal.next, s, insts);
		s.goal.next = clause;

		return(true); 
	    }
//   	    System.err.println("call: instant backtrack: " + unifier);
	    s.backtrack();

	}

//   	System.err.println("call: failure - " + arg0);
	s.cp.finished();
	return(false);
    }
}
