package prolog.pred;

import prolog.*;

/**
 * partition/4: Partition a list around a pivot.
 *
 * This is a fully deterministic, optimised to the max version for use
 * in quicksort.
 *
 * partition(list, pivot, lesser_list, greater_list)
 *
 * @author Peter Gammie
 */

public class partition_4 extends Pred {
    public partition_4() {
	functor = "partition";
	arity = 4;
    }

    public boolean unify(Term t, Prolog s) {
	Structured st = (Structured)t;
	Structured list = (Structured)st.args[0].getBinding();
	int pivot = ((NInteger)st.args[1].getBinding()).val;
	Variable out_le = (Variable)st.args[2];
	Variable out_gt = (Variable)st.args[3];

	while(true) {
// 	    System.out.println(list);

	    if(list.getFunctor().equals("[]")) {
		out_le.bind(list);
		out_gt.bind(list);
		s.push(out_le);
		s.push(out_gt);
		return(true);
	    }

	    Variable v = new Variable(null, 1);
	    Structured cons = new Structured(".", new Term[]{list.args[0], v});

	    if(((NInteger)list.args[0]).val <= pivot) {
		s.push(out_le);
		out_le.bind(cons);
		out_le = v;
	    } else {
		s.push(out_gt);
		out_gt.bind(cons);
		out_gt = v;
	    }

	    list = (Structured)list.args[1].getBinding();
	}
    }
}
