package prolog.pred;

import prolog.*;

/**
 * rule/2: Lookup a unifier for the first argument, and unify the second with
 * it's body represented as a list. e.g.:
 * <PRE>
 * a(x) :- b(y), c(x, y).
 *
 * rule(a(x), [b(y), c(x, y)]).
 * </PRE>
 * This is very similar in behaviour to clause/2, but saves some hassle with
 * the parser.
 *
 * @author Peter Gammie
 */
public class rule_2 extends CompiledClause {
    public rule_2() {
	super(true);
    }

    public boolean unify(Term t, Prolog s) {
	Compound arg, arg0;
	Term arg1;
	Clause cl;
	GoalList body;
	Head unifier;
	Compound bodyList = Prolog.endOfList;

	try {
	    arg = (Compound)t;
	    arg0 = (Compound)arg.args[0].getBinding();
	    arg1 = arg.args[1].getBinding();
	} catch(ClassCastException e) {
	    throw new RuntimeException("rule/2: invalid argument: " + t);
	}

	cl = s.cp.tmp != null ? ((ClauseList)s.cp.tmp).next : s.db.get(arg0);

	for(; cl != null; cl = cl.next) {
	    unifier = cl.h;

	    /* Instantiate the variables of the body. */
	    Term[] is = new Term[unifier.numVars];

// 	    System.err.println("rule/2: unifier = " + unifier);

	    if(unifier.unify(arg0, is, s)) {
		body = unifier.copyBody(null, is, s);

		/* Turn the body of this clause into a list. */
		for(Compound cur = null; body != null; body = body.next) {
		    if(cur != null) {
			Compound cur2 = cur;
			cur2.args[1] = cur = new Compound(Prolog.listFunctor,
							  new Term[]{body.g.copy(is, s),
									 Prolog.endOfList});
		    } else {
			bodyList = cur = new Compound(Prolog.listFunctor,
						      new Term[]{body.g.copy(is, s),
								     Prolog.endOfList});
		    }
		}

// 		System.err.println("rule/2: bodyList = " + bodyList);

		if(arg1.unify(bodyList, insts, s)) {
		    s.cp.tmp = cl;
		    return(true);
		}
	    } else {
		throw new RuntimeException("rule/2: broken on call: " + t);
	    }
	}
	s.cp.finished();
	return(false);
    }
}
