% -*- Prolog -*-
%
% N-queens solver.
%  from "The Art of Prolog" -- Leon Sterling, Ehud Shapiro, MIT Press.
%

% Queens is bound to a permutation of a list of numbers [1, 2, ..., N] that
% represents a solution to the N queens problem.

dynamic(count/1)!

%main :- queens(8, Queens).

%main :- count_queens.

count_queens :-
	assert(count(0)),
	queens(8, Queens),
	write(Queens), nl,
	count(X),
	retract(count(X)),
	is(X1, X + 1),
	assert(count(X1)),
	fail.
count_queens :-
	count(X),
	write("Number of solutions = "), write(X), nl,
	retract(count(X)).

% It's easier in prolog than native
not(X) :- call(X), !, fail.
not(X).

% Naive generate-and-test solution
queens(N, Qs) :-
	range(1, N, Ns),
	permutation(Ns, Qs),
	safe(Qs).

safe([Q|Qs]) :-
	safe(Qs),
	not(attack(Q, Qs)).
safe([]).

attack(X, Xs) :- attack(X, 1, Xs).

attack(X, N, [Y|Ys]) :-	is(X, Y + N).
attack(X, N, [Y|Ys]) :-	is(X, Y - N).
attack(X, N, [Y|Ys]) :- is(N1, N + 1), attack(X, N1, Ys).

permutation([], []) :- !.
permutation(Xs, [Z|Zs]) :-
	select(Z, Xs, Ys),
	permutation(Ys, Zs).

select(X, [X|Xs], Xs).
select(X, [Y|Ys], [Y|Zs]) :- select(X, Ys, Zs).

range(N, N, [N]) :- !.
range(M, N, [M|Ns]) :-
%	M < N,
	lt(M, N),
	is(M1, M + 1),
	range(M1, N, Ns).

% Place one queen at a time.
queens2(N, Qs) :-
	range(1, N, Ns),
	queens2(Ns, [], Qs).

queens2(UnplacedQs, SafeQs, Qs) :-
	select(Q, UnplacedQs, UnplacedQs1),
	not(attack(Q, SafeQs)),
	queens2(UnplacedQs1, [Q|SafeQs], Qs).
queens2([], Qs, Qs).
