% -*- Prolog -*-
%
% Bottom-up diagnosis of a false solution.
%  If A is a provable false instance, then Clause is a false clause in the
%  program. Bottom-up algorithm.
%    from "The Art of Prolog" -- Leon Sterling, Ehud Shapiro, MIT Press.
%

false_solution(A, Clause) :-
	solve(A, Proof),
	false_clause(Proof, Clause).

solve([], []).
solve([A|B], [ProofA|ProofB]) :-
	!, solve(A, ProofA), solve(B, ProofB).
solve(A, implies(A, builtin)) :- builtin(A), !, call(A).
solve(A, implies(A, Proof)) :- rule(A, B), solve(B, Proof).

false_clause([], ok).
false_clause([A|B], Clause) :-
	false_clause(A, ClauseA),
	check_conjunction(ClauseA, B, Clause).
false_clause(implies(A, B), Clause) :-
	false_clause(B, ClauseB),
	check_clause(ClauseB, A, B, Clause).

check_conjunction(ok, B, Clause) :-
	false_clause(B, Clause).
check_conjunction([implies(A, B1)], B, [implies(A, B1)]).

check_clause(ok, A, B, Clause) :-
	query_goal(A, Answer),
	check_answer(Answer, A, B, Clause).
check_clause([implies(A1, B1)], A, B, [implies(A1, B1)]).

check_answer(true, A, B, ok).
check_answer(false, A, B, [implies(A, B1)]) :-
	extract_body(B, B1).

extract_body([], []).
extract_body([implies(A, B)], A).
extract_body([implies(A, B)|Bs], [A|As]) :-
	extract_body(Bs, As).

query_goal(A, true) :-
	builtin(A).
query_goal(Goal, Answer) :-
	not(builtin(Goal)),
	write("Is the goal "), write(Goal), write(" true?"), nl,
	read(Answer).

% It's easier in prolog than native
not(X) :- call(X), !, fail.
not(X).

% append/3 - concat two lists in order
append([], X, X).
append([X|Xs], Y, [X|Z]) :- append(Xs, Y, Z).

% nrev/2 - naive reverse
nrev([],[]).
nrev([X|Xs], Zs) :- nrev(Xs, Ys), append(Ys, [X], Zs).

% Buggy insertion sort.
isort([X|Xs], Ys) :- isort(Xs, Zs), insert(X, Zs, Ys).
isort([], []).

insert(X, [Y|Ys], [X, Y|Ys]) :- ge(X, Y).
insert(X, [Y|Ys], [Y|Zs]) :- gt(X, Y), insert(X, Ys, Zs).
insert(X, [], [X]).

main :-
	false_solution(isort([3, 2, 1], X), C).
