% -*- Prolog -*-
%
% from "The Art of Prolog" - Sterling & Shapiro.
%
% grammar(X), translate(X, Y), assert_clauses(Y).
%
% $Id: ss_dcg.pro,v 1.1 2000/09/15 15:11:35 peteg Exp $

:- op(400, xfx, \).

/*	
    translate(Grammar,Program) :-
	Program is the Prolog equivalent of the context-free
	grammar Grammar.
*/
        translate([Rule|Rules],[Clause|Clauses]) :-
           translate_rule(Rule,Clause),
           translate(Rules,Clauses).
        translate([ ],[ ]).

/*
        translate_rule(GrammarRule,PrologClause) :-
           PrologClause is the Prolog equivalent of the grammar
           rule GrammarRule.
*/
	translate_rule((Lhs --> Rhs),(Head :- Body)) :-
	   translate_head(Lhs,Head,Xs\Ys),
	   translate_body(Rhs,Body,Xs\Ys),!.

        translate_head(A,A1,Xs) :-
           translate_goal(A,A1,Xs).

	translate_body((A,B),(A1,B1),Xs\Ys) :-
	   !, translate_body(A,A1,Xs\Xs1), translate_body(B,B1,Xs1\Ys).
        translate_body(A,A1,Xs) :-
           translate_goal(A,A1,Xs).

        translate_goal(A,A1,DList) :-
           nonterminal(A), functor(A1,A,1), arg(1,A1,DList).
        translate_goal(Terms,connect(Terms,S),S) :-
           terminals(Terms).

	nonterminal(A) :- atom(A).

	terminals(Xs) :- list(Xs).

	list([]).
	list([X|Xs]) :- list(Xs).

%     Program 18.9: Translating grammar rules to Prolog clauses

% Examples
:- dynamic(s/1).
:- dynamic(a/1).
:- dynamic(b/1).
:- dynamic(c/1).

s --> a, b, c.

a --> [a], a.
a --> [].
b --> [b], b.
b --> [].
c --> [c], c.
c --> [].

% usual findall/3 predicate.
:- dynamic(queue/1).

findall(X, Goal, _) :-
	call(Goal),
	assertz(queue(X)),
	fail.
findall(_, _, Xlist) :-
	assertz(queue(bottom)),
	collect(Xlist), !.

collect(L) :-
	retract(queue(X)), !,
	new_list(X, L).

new_list(bottom, []).
new_list(X, [X|L1]) :-
	collect(L1).

% Driver code.
grammar(X) :-
	findall( (A --> B), (A --> B), X).

assert_clauses([X|Xs]) :-
	assertz(X),
	assert_clauses(Xs).
assert_clauses([]).
