%\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

% Turbo Chess, by Frank Bergmann, V. 0.1
% SWI-Prolog version, by Manuel Hernandez Gutierrez
%
% (I have changed the data structures and the user's interaction,
% the actual is not for human beings!)
% April 28, 1998.
% Future directions: new data structures (bits?), transformation of the
% program, new evaluation function, new moves (castle?), the rol of
% the time, to "think" while is the opponent's turn,... etc., etc.
% With Prolog, all it's possible!

%//////////////////////////////////////////////////////////////////

tmove(X,Y).
% :- write(X),write(Y).

%--------------------------globals

rand(X):- X =< 10,!.
rand(X):- X >= 89,!.
rand(X):- 0 is X mod 10,!.
rand(X):- 9 is X mod 10,!.
	
exist(Sq,[X,_,_,_,_,_],p) :-  single(Sq,X).
exist(Sq,[_,X,_,_,_,_],r) :-  single(Sq,X).
exist(Sq,[_,_,X,_,_,_],n) :-  single(Sq,X).
exist(Sq,[_,_,_,X,_,_],b) :-  single(Sq,X).
exist(X,[_,_,_,_,[X],_],q).
exist(X,[_,_,_,_,_,[X]],k).

single(X,[X|_]).
single(X,[_|Rest]) :- single(X,Rest).

invert(black,white).
invert(white,black).

remove(X,[X|New],New):- !.
remove(X,[A|Old],[A|New]):-
	remove(X,Old,New).

for(I,I,_,I):- !.	
for(I,To,_,I):- I =\= To.
for(I,To,Step,Index):-
	I =\= To,
	I1 is I + Step,
	for(I1,To,Step,Index).	

end_for(I,I).	

%----------------Board (Adapted for Arity Prolog)



initfigur:-
        assertz(fb(0,0,'RW')),assertz(fb(1,0,'NW')),
        assertz(fb(2,0,'BW')),assertz(fb(3,0,'QW')),
        assertz(fb(4,0,'KW')),assertz(fb(5,0,'BW')),
        assertz(fb(6,0,'NW')),assertz(fb(7,0,'RW')),
        assertz(fb(0,1,'PW')),assertz(fb(1,1,'PW')),
        assertz(fb(2,1,'PW')),assertz(fb(3,1,'PW')),
        assertz(fb(4,1,'PW')),assertz(fb(5,1,'PW')),
        assertz(fb(6,1,'PW')),assertz(fb(7,1,'PW')),
        assertz(fb(0,6,'PB')),assertz(fb(1,6,'PB')),
        assertz(fb(2,6,'PB')),assertz(fb(3,6,'PB')),
        assertz(fb(4,6,'PB')),assertz(fb(5,6,'PB')),
        assertz(fb(6,6,'PB')),assertz(fb(7,6,'PB')),
        assertz(fb(0,7,'RB')),assertz(fb(1,7,'NB')),
        assertz(fb(2,7,'BB')),assertz(fb(3,7,'QB')),
        assertz(fb(4,7,'KB')),assertz(fb(5,7,'BB')),
        assertz(fb(6,7,'NB')),assertz(fb(7,7,'RB')),!.

closechess:-
        eraseall(figur(_,_)),
        eraseall(fb(_,_,_)),
        eraseall(stack(_,_,_)),
        eraseall(board(_,_)),
        eraseall(human(_)),
        eraseall(depth(_)),
        eraseall(top(_)),
	fail.
closechess:- write('Bye...').
	
who_vs_who:-
        tmove(1,1), write('Human(W) vs Human(B)   ( 1 )'),
        tmove(2,1), write('Human(W) vs Computer(B)( 2 )'),
        tmove(3,1),write('Computer(W) vs Human(B)( 3 )'),
	get_vs(I),
        save_color(I).
	
get_vs(I):-
	read(I),
	I > 0,I < 4,!.
get_vs(I):- get_vs(I).

save_color(1):-
	asserta(human(white)),
	assertz(human(black)),!.		
save_color(2):- asserta(human(white)),!.
save_color(3):- asserta(human(black)),!.	

screen_coo(X,Y,Xpos,Ypos):-
	  Xpos is X*5 + 10,
	  Ypos is 17 - Y*2.

putfigurall:-	
	fb(X,Y,Fig),
        screen_coo(X,Y,X3,Y3),
	  tmove(Y3,X3),
	  write(Fig),
	fail.
putfigurall.	

putfigur(X,Y,Fig):-
        screen_coo(X,Y,X3,Y3),
	tmove(Y3,X3), write(Fig),
	assertz(fb(X,Y,Fig)).

removefigur(X,Y):-
        screen_coo(X,Y,X3,Y3),
        tmove(Y3,X3), write('  '),
	retract(fb(X,Y,_)).

draw_board:-
	draw_hlines,
	draw_vlines,
	draw_left_index,
	draw_lower_index,
	!.
	
draw_hlines:-
	Xend is 48,
	Yend is 18,
	for(8,Xend,1,X),
	  for(2,Yend,2,Y),
            tmove(Y,X), write(''),
	  end_for(Y,Yend),
	end_for(X,Xend),!.

draw_vlines:-
	Xend is 48,
	Yend is 18,
	for(8,Xend,5,X),
	  for(2,Yend,1,Y),
            tmove(Y,X), write(''),
	  end_for(Y,Yend),
	end_for(X,Xend),!.

draw_left_index:-	
	for(8,1,-1,Index),
          screen_coo(-1,Index,Xpos,Ypos),
	  Y is Ypos + 2,
	  tmove(Y,Xpos), write(Index),
	end_for(Index,1).

draw_lower_index:-
	for(0,7,1,Index),
	  I is 1 + Index, %65
	  Xpos is 5*Index + 10,
	  tmove(20,Xpos), write(I),
	end_for(Index,7).

initboard:-
	who_vs_who,
	initfigur,
%        draw_board,
%        putfigurall,
	!.

%---------------Move generator

one_step(Sq,Dir,Next,Color,Pos):-
        Next is Sq + Dir,
        \+(rand(Next)),
        \+(occupied(Next,Color,Pos)).

call_mul(Sq,Dir,Next,Color,Pos):-
        Step is Sq + Dir,
        mul(Step,Dir,Next,Color,Pos).

mul(Sq,_,_,_,_):-
	rand(Sq),
	!,fail.
mul(Sq,_,_,Color,Pos):-
 	occupied(Sq,Color,Pos),
 	!,fail.
mul(Sq,_,Sq,Color,Pos):-
	invert(Color,Oppo),
	occupied(Sq,Oppo,Pos),!.
mul(Sq,_,Sq,_,_).
mul(Sq,Direction,Next,Color,Pos):-
	Step is Sq + Direction,
        mul(Step,Direction,Next,Color,Pos).
	
half(pos(Half,_),Half,white).
half(pos(_,Half),Half,black).

add_half(pos(_,Y),Half,white,pos(Half,Y)).
add_half(pos(X,_),Half,black,pos(X,Half)).

occupied(Sq,white,pos(Stones,_)):-
	exist(Sq,Stones,_).	
occupied(Sq,black,pos(_,Stones)):-
	exist(Sq,Stones,_).	

free(Sq,Pos):-
        \+(occupied(Sq,white,Pos)),
        \+(occupied(Sq,black,Pos)),
        \+(rand(Sq)).

poss_move(r,10).
poss_move(r,-10).
poss_move(r,1).
poss_move(r,-1).
poss_move(b,9).
poss_move(b,11).
poss_move(b,-9).
poss_move(b,-11).
poss_move(n,19).
poss_move(n,21).
poss_move(n,8).
poss_move(n,12).
poss_move(n,-8).
poss_move(n,-12).
poss_move(n,-19).
poss_move(n,-21).
poss_move(q,X):-
	poss_move(r,X).
poss_move(q,X):-
	poss_move(b,X).
poss_move(k,X):-
	poss_move(q,X).
	
bauermove(From,white,Pos,To):-
	To is From + 9,
	occupied(To,black,Pos).
bauermove(From,white,Pos,To):-
	To is From + 10,
        free(To,Pos).
bauermove(From,white,Pos,To):-
	To is From + 11,
	occupied(To,black,Pos).
bauermove(From,white,Pos,To):-
	To is From + 20,
	Over is From + 10,
        free(To,Pos),
        free(Over,Pos),
	Row is From / 10,
	Row > 2,
	Row < 3.
bauermove(From,black,Pos,To):-
	To is From - 9,
	occupied(To,white,Pos).
bauermove(From,black,Pos,To):-
	To is From - 10,
        free(To,Pos).
bauermove(From,black,Pos,To):-
	To is From - 11,
	occupied(To,white,Pos).
bauermove(From,black,Pos,To):-
	To is From - 20,
	Over is From - 10,
        free(To,Pos),
        free(Over,Pos),
	Row is From / 10,
	Row > 7,
	Row < 8.

longmove(From,Color,Typ,Pos,To):-
	poss_move(Typ,Dir),
        call_mul(From,Dir,To,Color,Pos).
shortmove(From,Color,Typ,Pos,To):-
	poss_move(Typ,Dir),
	one_step(From,Dir,To,Color,Pos).
	
all_moves(Color,Pos,move(From,To)):-
	half(Pos,[Bauern,_,_,_,_,_],Color),
	single(From,Bauern),
	bauermove(From,Color,Pos,To).
all_moves(Color,Pos,move(From,To)):-
	half(Pos,[_,Rookies,_,_,_,_],Color),
	single(From,Rookies),
	longmove(From,Color,r,Pos,To).
all_moves(Color,Pos,move(From,To)):-
	half(Pos,[_,_,Knights,_,_,_],Color),
	single(From,Knights),
	shortmove(From,Color,n,Pos,To).
all_moves(Color,Pos,move(From,To)):-
	half(Pos,[_,_,_,Bishies,_,_],Color),
	single(From,Bishies),
	longmove(From,Color,b,Pos,To).
all_moves(Color,Pos,move(From,To)):-
	half(Pos,[_,_,_,_,Queenies,_],Color),
	single(From,Queenies),
	longmove(From,Color,q,Pos,To).
all_moves(Color,Pos,move(King,To)):-
	half(Pos,[_,_,_,_,_,[King]],Color),
	shortmove(King,Color,k,Pos,To).

%---------------------stack

init_stack :-
	asserta(top(0)).

push(Move,Value) :-
	retract(top(Old)),
	New is Old+1,
	asserta(top(New)),
	asserta(stack(Move,Value,New)),!.
	
pull(Move,Value) :-
	retract(top(Old)),!,
	New is Old-1,
	asserta(top(New)),
	retract(stack(Move,Value,Old)),!.
	
get(Move,Value,Depth) :-
	top(Top),
	Act is Top-Depth,
	stack(Move,Value,Act),!.

get_0(Move,Value) :-
	top(Top),
	stack(Move,Value,Top),!.
	
replaces(Move,Value) :-	
	top(Top),
	retract(stack(_,_,Top)),
	asserta(stack(Move,Value,Top)),!.

%------------------ Position Evaluation

compensate(white,15).
compensate(black,-15).
	
worst_value(white, -10000). 
worst_value(black, 10000).

winning(white, 9000).
winning(black,-9000).	
		
count_halfst([P,R,N,B,Q,[_]],Color,Value) :-
        pos_count(p,P,Color,V1),
        pos_count(r,R,Color,V2),
        pos_count(b,B,Color,V3),
        pos_count(n,N,Color,V4),
        pos_count(q,Q,Color,V5),
        double_bonus(R,D1),
        double_bonus(N,D2),
        double_bonus(B,D3),
	Value is V1+V2+V3+V4+V5+30*(D1+D2+D3).

double_bonus([_,_],1) :- !.
double_bonus(_,0).

pos_count(_,[],_,0) :- !.
pos_count(Type,[Sq|Rest],Color,Value) :-
	pos_count(Type,Rest,Color,V2),
	pos_value(Type,Sq,Color,V1),	
	Value is V1+V2,!.
		
pos_value(Type,Sq,black,Value) :-
	Rel_Sq is 99-Sq,
	pos_value(Type,Rel_Sq,white,Value),
	!.   
pos_value(b,Pos,white,Value) :-
	row(Pos,Row),
	row_value(b,Row,Value),!.
pos_value(q,Pos,white,Value) :-
	row(Pos,Row),
	row_value(q,Row,Value),!.	
pos_value(p,34,white,127) :- !.
pos_value(p,35,white,127) :- !.
pos_value(p,44,white,131) :- !.
pos_value(p,45,white,131) :- !.
pos_value(p,54,white,131) :- !.
pos_value(p,55,white,131) :- !.
pos_value(p,_,white,100) :- !.
pos_value(k,11,white,900) :- !. %30?
pos_value(k,12,white,900) :- !.
pos_value(k,13,white,30) :- !.
pos_value(k,17,white,900) :- !.
pos_value(k,18,white,30) :- !.
pos_value(r,_,white,450) :- !.
pos_value(n,Pos,white,Value) :-
	row_line(Pos,Row,Line),
	row_value(n,Row,V1),
	line_value(n,Line,V2),
	Value is V1+V2,!.

row_value(n,2,320) :- !.
row_value(n,3,321) :- !.
row_value(n,4,348) :- !.
row_value(n,5,348) :- !.
row_value(n,6,376) :- !.
row_value(n,7,376) :- !.
row_value(n,_,290) :- !.
row_value(b,1,300) :- !.
row_value(b,2,329) :- !.
row_value(b,3,329) :- !.
row_value(b,_,330) :- !.
row_value(q,1,850) :- !.
row_value(q,_,876) :- !.

line_value(n,1,0) :- !.
line_value(n,8,0) :- !.
line_value(n,_,10) :- !.	

row(Pos,Row) :-
	Row is Pos // 10.
row_line(Pos,Row,Line) :-
	Row is Pos // 10,
	Line is Pos mod 10.

generate(Move,Color_,Old,New,Hit):-
	all_moves(Color_,Old,Move),
	make_move(Color_,Old,Move,New,Hit).

%-----------------------------Tree Management


newdepth(Depth,hit,NewDepth) :-
	top(X),
	X<4,
	NewDepth is 1,!.
newdepth(Depth,_,NewDepth) :-
	NewDepth is Depth-1,!.	

get_best(Pos,Color,Depth,Alpha,Beta) :-
	invert(Color,Op),
	generate(Move,Color,Pos,Neu_Pos,Hit),
	newdepth(Depth,Hit,New_Depth),
	new_alpha_beta(Color,Alpha,New_Alpha,Beta,New_Beta),
	eval(Neu_Pos,Op,Value,_,New_Depth,New_Alpha,New_Beta),
	compares(Move,Value,Color),
	cutting(Value,Color,Alpha,Beta),
	!,
	fail.
	
new_alpha_beta(white,Alpha,New_Alpha,Beta,Beta) :-
	get_0(A,Value),
	Value>Alpha,
	New_Alpha is Value,!.
new_alpha_beta(black,Alpha,Alpha,Beta,New_Beta) :-
	get_0(B,Value),
	Value<Beta,
	New_Beta is Value,!.
new_alpha_beta(_,Alpha,Alpha,Beta,Beta).
	
compares(_,Value,white) :-
	get_0(C,Old),
	Old>=Value,
	!.
compares(_,Value,black) :-
	get_0(D,Old),
	Old=<Value,
	!.
compares(Move,Value,_) :-
	replaces(Move,Value).

cutting(Value,white,_,Beta) :-
	Beta<Value.
cutting(Value,black,Alpha,_) :-
	Alpha>Value.

eval(pos([_,_,_,_,_,[]],_),_,Value,move(0,0),_,_,_) :-
	winning(black,Value),!.
eval(pos(_,[_,_,_,_,_,[]]),_,Value,move(0,0),_,_,_) :-
	winning(white,Value),!.
eval(pos(W,B),Color,Value,move(0,0),0,_,_) :-
	count_halfst(W,white,X),
	count_halfst(B,black,Y),
	compensate(Color,Z),
	Value is X-Y+Z,
	!.
eval(Pos,Color,Value,Move,Depth,Alpha,Beta) :-
	worst_value(Color,Worst),
	push(move(0,0),Worst),	
        \+(get_best(Pos,Color,Depth,Alpha,Beta)),
	pull(Move,Value),
	!.

%-------------------Control Management

enter(Pos,Color,Move) :-
	human(Color),
	tmove(0,0),
        read(From),            %   |    coordinates: x. y.
        read(To),              %   | x
        Move=move(From,To),    %   |     y
        importmove(Move),      %   |-----------
	!.

enter(Pos,Color,Move) :-	
	depth(Depth),!,
	worst_value(white,Alpha),
	worst_value(black,Beta),
	eval(Pos,Color,Value,Move,Depth,Alpha,Beta),
	importmove(Move),!.	
	
play(GrundPos,Anzug) :-
        asserta(board(GrundPos,Anzug)),
	repeat,
        retract(board(Pos,Color)),
          write(Pos),
	  enter(Pos,Color,Move),
	  make_move(Color,Pos,Move,New,_),
	  invert(Color,Op),
          asserta(board(New,Op)),
	fail.
play(_,_).

%----------------- Global Predicates

change(Old,Color,From,To,New):-
	half(Old,Half,Color),
	exist(From,Half,Type),
	extract(Half,Type,Liste),
	remove(From,Liste,Templist),
	combine(Half,Type,[To|Templist],Newhalf),
	add_half(Old,Newhalf,Color,New).

kill(Old,Color,Sq,New):- 
	half(Old,Half,Color),
	exist(Sq,Half,Type),
	extract(Half,Type,Liste),
	remove(Sq,Liste,Newlist),
	combine(Half,Type,Newlist,Newhalf),
	add_half(Old,Newhalf,Color,New).
	
extract([X,_,_,_,_,_],p,X).
extract([_,X,_,_,_,_],r,X).
extract([_,_,X,_,_,_],n,X).
extract([_,_,_,X,_,_],b,X).
extract([_,_,_,_,X,_],q,X).
extract([_,_,_,_,_,X],k,X).

combine([_,B,C,D,E,F],p,N,[N,B,C,D,E,F]).
combine([A,_,C,D,E,F],r,N,[A,N,C,D,E,F]).
combine([A,B,_,D,E,F],n,N,[A,B,N,D,E,F]).
combine([A,B,C,_,E,F],b,N,[A,B,C,N,E,F]).
combine([A,B,C,D,_,F],q,N,[A,B,C,D,N,F]).
combine([A,B,C,D,E,_],k,N,[A,B,C,D,E,N]).
	
%-------------------------MakeMove Routines

check_00(Old,white,15,17,New) :-
	Old=pos([_,_,_,_,_,[15]],_),
	change(Old,white,18,16,New),!.
check_00(Old,white,15,13,New) :-
	Old=pos([_,_,_,_,_,[15]],_),
	change(Old,white,11,14,New),!.
check_00(Old,black,85,87,New) :-
	Old=pos(_,[_,_,_,_,_,[85]]),
	change(Old,black,88,86,New),!.
check_00(Old,black,85,83,New) :-
	Old=pos(_,[_,_,_,_,_,[85]]),
	change(Old,black,81,84,New),!.
check_00(Old,_,_,_,Old).

make_move(Color_,Old,move(From,To),New,hit):-
	invert(Color_,Oppo),
	kill(Old,Oppo,To,Temp),
	change(Temp,Color_,From,To,New),!.
make_move(Color_,Old,move(From,To),New,nohit):-
	check_00(Old,Color_,From,To,Temp),
	change(Temp,Color_,From,To,New),!.

move_1(X1,Y1,X2,Y2):-
	fb(X1,Y1,Fig),
	removefigur(X1,Y1),
	putfigur(X2,Y2,Fig).

importmove(move(From,To)):-		% user interface
	intoxy(From,X1,Y1),
	intoxy(To,X2,Y2),
	move_1(X1,Y1,X2,Y2).
	
intoxy(A,X,Y):-	X is (A mod 10) - 1, Y is (A // 10) - 1.
%Aqui va ilegal.
%	GOAL Section
%****************************************************************

initpos(pos(H1,H2)):-
        Pw = [21,22,23,24,25,26,27,28],
        H1 = [Pw,[11,18],[12,17],[13,16],[14],[15]],
        Pb = [71,72,73,74,75,76,77,78],
        H2 = [Pb,[81,88],[82,87],[83,86],[84],[85]].

eraseall(X) :- retract(X),
        fail.
eraseall(_).

main:-
	eraseall(stack(_,_,_)),
	eraseall(fb(_,_,_)),
	eraseall(human(_)),
	eraseall(depth(_)),
        eraseall(board(_,_)),
	initpos(Pos),
        asserta(depth(3)),
	init_stack,
	initboard,
	play(Pos,white),
	closechess.	
			

