% -*- Prolog -*-
%
% Turing Machine implementation.
%  Peter Gammie, peteg@cse.unsw.edu.au, April 1999.

% The tape is represented as two lists (stacks), with the current square being
% the head of the second one.

%** yes
%  _0 = t([], [0])
%  _1 = 1
%  _2 = 0
%  _3 = 1
%  _4 = r
%  _5 = 2
%  _6 = 0
%  _7 = t([], [1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])

unify(X, X).

main :-
	tape_new(Tape),
	state(State, _, _, _, _), !,
	tm(State, Tape, EState, ETape),
	write("Start state: "), write(State), nl,
	write("End state: "), write(EState), nl.

% Main loop
tm(State, Tape, EState, ETape) :-
	% Get the currently scanned symbol.
	tape_cs(Tape, Symbol),
	% Select a state 
	state(State, Symbol, Symbol1, Move, State1),
	tape_update_move(Move, Tape, Symbol1, Tape1),
	!, tm(State1, Tape1, EState, ETape).
tm(State, Tape, State, Tape).


% Tape manipulation
tape_new(t([], [0])).
tape_cs(t(_, [R|Rs]), R).
tape_update_move(l, t([L|Ls], [R|Rs]), Symbol, t(Ls, [L, Symbol|Rs])).
tape_update_move(l, t([], [R|Rs]), Symbol, t([], [0, Symbol|Rs])).
tape_update_move(r, t(Left, [R]), Symbol, t([Symbol|Left], [0])).
tape_update_move(r, t(Left, [R|Rs]), Symbol, t([Symbol|Left], Rs)).
tape_update_move(n, t(Left, [R|Rs]), Symbol, t(Left, [Symbol|Rs])).

% Tuples

% 4-state busy beaver
state(1,0,1,r,2).
state(1,1,1,l,2).
state(2,0,1,l,1).
state(2,1,0,l,3).
state(3,0,1,n,0).
state(3,1,1,l,4).
state(4,0,1,r,4).
state(4,1,0,r,1).

% 5-state candidate bb (4098)
%state(1,0,1,L,2).
%state(1,1,1,R,3).
%state(2,0,1,L,3).
%state(2,1,1,L,2).
%state(3,0,1,L,4).
%state(3,1,0,R,3).
%state(4,0,1,R,1).
%state(4,1,1,R,4).
%state(5,0,1,L,0).
%state(5,1,0,R,1).
