#!/usr/local/bin/perl -w
#
# test.pl: Tests for interpreter/compiler correctness.
#
# syntax: test [prolog flags] x, where x is:
#     1. the test number, or
#     2. a range of test numbers a-b, or
#     3. "-a" to run all tests.
#
# picks up input files "x" where x is the test number, diffs the standard
# output against "x.out".
#
# $Id: test.pl,v 1.7 2000/09/15 15:11:35 peteg Exp $
#
use strict;

my $path;
my $prolog;# = "/home/peteg/src/prolog_old/run";
my $prolog_args;

MAIN: {
  die "Usage: $0 [prolog flags] testnumber\n" if $#ARGV < 0;

  # Unbuffered output so we know what's looping/crashing/etc.
  $| = 1;

  $path = `dirname $0`;
  chomp $path;

  if(!defined $prolog) {
    $prolog = "$path/../run";
  }

  my $test = pop(@ARGV);
  $prolog_args = defined @ARGV ? join(" ", @ARGV) : "";

  if($test eq "-a") {
    my @tests = glob("$path/[0-9][0-9][0-9]");

    print "all tests: @tests \n";

    foreach my $test (@tests) {
      exit if !run_test(substr($test, -3));
    }
  } else {
    if($test =~ /(\d+)-(\d+)?/) {
      my $upper_limit = $2;

      if(!defined $2) {
	$upper_limit = 1000;
      }

      # Run a range of tests
      for(my $i = $1; $i <= $upper_limit; $i++) {
	exit if !run_test($i);
      }
    } else {
      # Run a single test
      run_test($test);
    }
  }
}

sub run_test {
  my $test        = "$path/" . sprintf("%03d", $_[0]);
  my $test_out    = $test . ".out";
  my $test_diff   = $test . ".diff";
  my $test_stderr = $test . ".stderr";
  my $test_stdout = $test . ".stdout";

  if(!(-e $test)) {
    # Silently ignore missing tests
    #print "ignore test: $test\n";

    return 1;
  }

  print "Test $test... ";

#  print "$prolog $prolog_args $test\n";
  `$prolog $prolog_args $test < /dev/null 2> $test_stderr 1> $test_stdout`;

  # diff the stdout against the .out file
  `diff $test_out $test_stdout 2>&1 > $test_diff`;

  my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
      $atime,$mtime,$ctime,$blksize,$blocks) = stat($test_diff);

  if($size > 0) {
    print "failed. **\n";

    return 0;
  } else {
    print "passed.\n";
    unlink ($test_stderr,$test_stdout,$test_diff);

    return 1;
  }
}
